package com.alibaba.fastsql.funtions;

import com.alibaba.fastsql.DbType;
import com.alibaba.fastsql.sql.ast.SQLArrayDataType;
import com.alibaba.fastsql.sql.ast.SQLDataType;
import com.alibaba.fastsql.sql.ast.SQLDataTypeImpl;
import com.alibaba.fastsql.sql.ast.SQLMapDataType;
import com.alibaba.fastsql.sql.ast.expr.*;
import org.codehaus.janino.Java;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Map;

public class FunctionRepository {
    private final DbType dbType;
    private Map<Long, FunctionDef> functions = new LinkedHashMap<Long, FunctionDef>();

    public FunctionRepository(DbType dbType) {
        this.dbType = dbType;
    }

    public final static FunctionRepository MY_SQL = new FunctionRepository(DbType.mysql);

    private static SQLDataType[] dataType(Type[] types) {
        SQLDataType[] dataTypes = new SQLDataType[types.length];

        for (int i = 0; i < types.length; i++) {
            dataTypes[i] = dataType(types[i]);
        }

        return dataTypes;
    }

    private static SQLDataType dataType(Type type) {
        if (String.class == type) {
            return SQLCharExpr.DATA_TYPE;
        }

        if (int.class == type || long.class == type) {
            return SQLIntegerExpr.DATA_TYPE;
        }

        if (double.class == type) {
            return SQLNumberExpr.DATA_TYPE_DOUBLE;
        }

        if (Date.class == type || java.sql.Date.class == type) {
            return SQLDateExpr.DATA_TYPE;
        }

        if (Timestamp.class == type) {
            return SQLTimestampExpr.DATA_TYPE;
        }

        if (Time.class == type) {
            return SQLTimeExpr.DATA_TYPE;
        }

        if (String[].class == type) {
            return SQLArrayDataType.ARRYA_CHAR;
        }

        if (byte[].class == type) {
            return new SQLDataTypeImpl(SQLDataTypeImpl.Constants.VARBINARY);
        }

        if (type instanceof ParameterizedType) {
            ParameterizedType paramType = (ParameterizedType) type;
            Type[] argTypes = paramType.getActualTypeArguments();
            if (argTypes.length == 2
                    && argTypes[0] == String.class
                    && argTypes[1] == String.class) {
                return SQLMapDataType.MAP_CHAR_CHAR;
            }
        }

        return null;
    }

    static {
        MY_SQL.registerFunction(StringFunctions.class);
        MY_SQL.registerFunction(DateFunctions.class);
        MY_SQL.registerFunction(MathFunctions.class);
    }

    public void registerFunction(String name, SQLDataType returnType, SQLDataType... parameterTypes) {
        registerFunction(name, false, returnType, parameterTypes);
    }

    public void registerFunction(String name, boolean varArg, SQLDataType returnType, SQLDataType... parameterTypes) {
        FunctionDef func = new FunctionDef(name, varArg, returnType, parameterTypes);
        functions.put(func.nameHash64(), func);
    }

    public FunctionDef find(long hash) {
        return functions.get(hash);
    }


    public void registerFunction(Class clazz) {
        Method[] methods = clazz.getMethods();
        for (Method method : methods) {
            if (!Modifier.isStatic(method.getModifiers())) {
                continue;
            }

            if (method.getDeclaringClass() == Object.class) {
                continue;
            }

            String name = method.getName();
            Type returnType = method.getGenericReturnType();
            Type[] paramTypes = method.getGenericParameterTypes();

            boolean isVarArgs = method.isVarArgs();

            if (isVarArgs) {
                Type lastType = paramTypes[paramTypes.length - 1];
                Type componentType = ((Class) lastType).getComponentType();
                paramTypes[paramTypes.length - 1] = componentType;
            }
            registerFunction(name, isVarArgs, dataType(returnType), dataType(paramTypes));
        }
    }
}
