package com.alibaba.fastsql.sql.optimizer.rules;

import com.alibaba.fastsql.sql.SQLUtils;
import com.alibaba.fastsql.sql.ast.SQLExpr;
import com.alibaba.fastsql.sql.ast.SQLName;
import com.alibaba.fastsql.sql.ast.SQLObject;
import com.alibaba.fastsql.sql.ast.expr.*;
import com.alibaba.fastsql.sql.ast.statement.*;
import com.alibaba.fastsql.sql.dialect.mysql.ast.statement.MySqlSelectQueryBlock;

import java.util.ArrayList;
import java.util.List;

public class DecorticateExistsWithAGG extends OptimizerVisitor {
    private int tableALiasSeed = 1;

    protected String genTableAlias() {
        return "SM_T_" + (tableALiasSeed++) + "_";
    }

    public boolean visit(SQLExistsExpr x) {
        SQLSelect select = x.getSubQuery();

        SQLSelectQueryBlock queryBlock = select.getQueryBlock();
        if (queryBlock == null) {
            return true;
        }

        SQLSelectGroupByClause groupBy = queryBlock.getGroupBy();
        if (groupBy == null || groupBy.getItems().size() == 0) {
            return true;
        }

        SQLTableSource from = queryBlock.getFrom();
        if (from instanceof SQLJoinTableSource) {
            return true;
        }

        SQLExpr where = queryBlock.getWhere();
        if (where == null) {
            return true;
        }

        if (x.isNot()) {
            return true; // TODO
        }

        final long aliasHashCode64 = from.aliasHashCode64();

        List<SQLExpr> items = SQLBinaryOpExpr.split(where, SQLBinaryOperator.BooleanAnd);

        List<SQLBinaryOpExpr> joinConditions = new ArrayList<SQLBinaryOpExpr>();

        for (SQLExpr item : items) {
            if (item instanceof SQLBinaryOpExpr) {
                SQLBinaryOpExpr binaryOpItem = (SQLBinaryOpExpr) item;

                if (!binaryOpItem.isBothName()) {
                    continue;
                }

                if (binaryOpItem.getOperator() != SQLBinaryOperator.Equality) {
                    return true;
                }

                SQLExpr left = binaryOpItem.getLeft();
                if (left instanceof SQLIdentifierExpr) {
                    return true; // TODO
                }

                SQLExpr right = binaryOpItem.getRight();
                if (right instanceof SQLIdentifierExpr) {
                    return true; // TODO
                }

//                SQLExpr leftOwner = ((SQLPropertyExpr) binaryOpItem.getLeft()).getOwner();
//                SQLExpr rightOener = ((SQLPropertyExpr) binaryOpItem.getRight()).getOwner();

                joinConditions.add(binaryOpItem);
            }
        }

        List<SQLExpr> array = new ArrayList<SQLExpr>();
        SQLListExpr selectArray = new SQLListExpr();

        List<SQLBinaryOpExpr> replacedConditions = new ArrayList<SQLBinaryOpExpr>();

        for (SQLBinaryOpExpr condition : joinConditions) {
            SQLPropertyExpr left = (SQLPropertyExpr) condition.getLeft();
            SQLPropertyExpr right = (SQLPropertyExpr) condition.getRight();

            if (!(left.getOwner() instanceof SQLIdentifierExpr)) {
                return true;
            }

            if (!(right.getOwner() instanceof SQLIdentifierExpr)) {
                return true;
            }

            SQLIdentifierExpr leftOwner = (SQLIdentifierExpr) left.getOwner();
            SQLIdentifierExpr rightOwner = (SQLIdentifierExpr) right.getOwner();

            if (leftOwner.nameHashCode64() == aliasHashCode64 && rightOwner.nameHashCode64() == aliasHashCode64) {
                return true;
            }

            if (leftOwner.nameHashCode64() != aliasHashCode64 && rightOwner.nameHashCode64() != aliasHashCode64) {
                return true;
            }

            if (leftOwner.nameHashCode64() == aliasHashCode64 && rightOwner.nameHashCode64() != aliasHashCode64) {
                selectArray.addItem(left.clone());
                array.add(right);
                replacedConditions.add(condition);
            } else if (leftOwner.nameHashCode64() != aliasHashCode64 && rightOwner.nameHashCode64() == aliasHashCode64) {
                array.add(left);
                selectArray.addItem(right.clone());
                replacedConditions.add(condition);
            }
        }

        SQLIdentifierExpr parentTable = null;
        for (SQLExpr item : array) {
            SQLPropertyExpr propertyItem = (SQLPropertyExpr) item;

            if (parentTable == null) {
                parentTable = (SQLIdentifierExpr) propertyItem.getOwner();
            } else {
                if (!parentTable.equals(propertyItem.getOwner())) {
                    return true;
                }
            }
        }

        SQLSelectQueryBlock inQuery = queryBlock.clone();
        inQuery.getSelectList().clear();
        for (SQLExpr item : SQLBinaryOpExpr.split(inQuery.getWhere(), SQLBinaryOperator.BooleanAnd)) {
            if (item instanceof SQLBinaryOpExpr && ((SQLBinaryOpExpr) item).isBothName()) {
                replaceInParent(item, null);
            }
        }
        inQuery.setDistinct();

        SQLObject parent = x.getParent();
        if (parent == null) {
            return true;
        }

        SQLSelectQueryBlock parentQuery = null;

        if (parent.getParent() instanceof SQLSelectQueryBlock) {
            parentQuery = (SQLSelectQueryBlock) parent.getParent();
        }

        if (parentQuery == null) {
            return true;
        }

        SQLTableSource parentFrom = parentQuery.getFrom();
        SQLExpr parentWhere = parentQuery.getWhere();
        SQLTableSource parentTableSource = parentFrom.findTableSource(parentTable.nameHashCode64());
        if (parentTableSource == null) {
            return true;
        }

        List<SQLExpr> matchParentFilters = new ArrayList<SQLExpr>();
        List<SQLExpr> parentFilters = SQLBinaryOpExpr.split(parentWhere, SQLBinaryOperator.BooleanAnd);
        for (SQLExpr item : parentFilters) {
            if (item instanceof SQLBinaryOpExpr) {
                SQLBinaryOpExpr parentFilter = (SQLBinaryOpExpr) item;
                SQLExpr left = parentFilter.getLeft();
                SQLExpr right = parentFilter.getRight();

                SQLPropertyExpr name = null;
                SQLLiteralExpr value = null;
                if (left instanceof SQLPropertyExpr && right instanceof SQLLiteralExpr) {
                    name = (SQLPropertyExpr) left;
                    value = (SQLLiteralExpr) right;
                } else if (right instanceof SQLPropertyExpr && left instanceof SQLLiteralExpr) {
                    name = (SQLPropertyExpr) right;
                    value = (SQLLiteralExpr) left;
                }

                if (name == null || value == null) {
                    continue;
                }

                if (name.getOwner().equals(parentTable)) {
                    matchParentFilters.add(parentFilter);
                }
            } else if (item instanceof SQLBetweenExpr) {
                SQLBetweenExpr betweenExpr = (SQLBetweenExpr) item;

                if (betweenExpr.getTestExpr() instanceof SQLPropertyExpr
                        && betweenExpr.getBeginExpr() instanceof SQLLiteralExpr
                        && betweenExpr.getEndExpr() instanceof SQLLiteralExpr)
                {
                    SQLPropertyExpr name = (SQLPropertyExpr) betweenExpr.getTestExpr();
                    if (name.getOwner().equals(parentTable)) {
                        matchParentFilters.add(item);
                    }
                }
            } else if (item instanceof SQLInListExpr) {
                SQLInListExpr in = (SQLInListExpr) item;
                if (in.getExpr() instanceof SQLPropertyExpr) {
                    SQLPropertyExpr name = (SQLPropertyExpr) in.getExpr();
                    if (name.getOwner().equals(parentTable)) {
                        matchParentFilters.add(item);
                    }
                }
            }
        }

        {
            String subQueryJoinToAlias = genTableAlias();
            SQLTableSource subQueryJoinTo = parentTableSource.clone();
            subQueryJoinTo.setAlias(subQueryJoinToAlias);

            String subQueryJoinToSubQueryAlias = genTableAlias();
            MySqlSelectQueryBlock subQueryJoinToSubQuery = new MySqlSelectQueryBlock();
            subQueryJoinToSubQuery.setDistinct();
            subQueryJoinToSubQuery.setFrom(subQueryJoinTo);
            for (SQLExpr item : selectArray.getItems()) {
                if (item instanceof SQLPropertyExpr) {
                    subQueryJoinToSubQuery.addSelectItem(new SQLIdentifierExpr(((SQLPropertyExpr) item).getName()));
                } else if (item instanceof SQLIdentifierExpr) {
                    subQueryJoinToSubQuery.addSelectItem(item.clone());
                } else {
                    return false;
                }
            }

            SQLJoinTableSource join = new SQLJoinTableSource(
                    inQuery.getFrom()
                    , SQLJoinTableSource.JoinType.INNER_JOIN
                    , new SQLSubqueryTableSource(subQueryJoinToSubQuery, subQueryJoinToSubQueryAlias)
                    , null
            );

            String leftAlias = inQuery.getFrom().computeAlias();
            for (int i = 0; i < array.size(); i++) {
                SQLExpr arrayItem = array.get(i).clone();
                if (arrayItem instanceof SQLPropertyExpr) {
                    ((SQLPropertyExpr) arrayItem).setOwner(leftAlias);
                } else if (arrayItem instanceof SQLIdentifierExpr) {
                    arrayItem = new SQLPropertyExpr(leftAlias, ((SQLIdentifierExpr) arrayItem).getName());
                } else {
                    return false;
                }

                SQLPropertyExpr subQueryColumn = new SQLPropertyExpr(subQueryJoinToSubQueryAlias, ((SQLName) selectArray.getItems().get(i)).getSimpleName());
                join.addCondition(
                        new SQLBinaryOpExpr(arrayItem, SQLBinaryOperator.Equality, subQueryColumn));

                inQuery.getGroupBy().addItem(i, subQueryColumn.clone());
//                inQuery.addSelectItem(arrayItem.clone());
                inQuery.addSelectItem(subQueryColumn.clone());
            }
            inQuery.setFrom(join);

            for (SQLExpr item : matchParentFilters) {
                SQLExpr subQueryJoinToFilter = item.clone();
                SQLPropertyExpr name = null;
                if (item instanceof SQLBinaryOpExpr) {
                    SQLBinaryOpExpr binaryOpExpr = (SQLBinaryOpExpr) subQueryJoinToFilter;

                    SQLExpr left = binaryOpExpr.getLeft();
                    SQLExpr right = binaryOpExpr.getRight();

                    if (left instanceof SQLPropertyExpr) {
                        name = (SQLPropertyExpr) left;
                    } else if (right instanceof SQLPropertyExpr) {
                        name = (SQLPropertyExpr) right;
                    }
                } else if (item instanceof SQLBetweenExpr) {
                    name = (SQLPropertyExpr) ((SQLBetweenExpr) item).getTestExpr();
                } else if (item instanceof SQLInListExpr) {
                    name = (SQLPropertyExpr) ((SQLBetweenExpr) item).getTestExpr();
                } else {
                    throw new IllegalStateException();
                }

                name.setOwner(subQueryJoinToAlias);
                subQueryJoinToSubQuery.addWhere(subQueryJoinToFilter);
            }

        }

        SQLSubqueryTableSource tableSource = new SQLSubqueryTableSource(inQuery);
        String tableSourceAlias = genTableAlias();
        tableSource.setAlias(tableSourceAlias);

        SQLJoinTableSource join = new SQLJoinTableSource(parentFrom, SQLJoinTableSource.JoinType.INNER_JOIN, tableSource, null);
        for (int i = 0; i < array.size(); i++) {
            SQLPropertyExpr subQueryColumn = new SQLPropertyExpr(tableSourceAlias, ((SQLName) selectArray.getItems().get(i)).getSimpleName());
            join.addCondition(
                    new SQLBinaryOpExpr(array.get(i).clone(), SQLBinaryOperator.Equality, subQueryColumn)
            );
        }

        for (SQLBinaryOpExpr item : joinConditions) {
            inQuery.removeCondition(item);
        }

        boolean replaced = replaceInParent(x, null);
        if (!replaced) {
            return true;
        }

//        for (SQLExpr item : selectArray.getItems()) {
//            inQuery.addSelectItem(item);
//        }


        parentQuery.setFrom(join);

//
//        if (selectArray.getItems().size() == 1) {
//            inQuery.addSelectItem(selectArray.getItems().get(0));
//        } else {
//            inQuery.addSelectItem(selectArray);
//        }
//
//
//        SQLInSubQueryExpr inSubQueryExpr
//                = new SQLInSubQueryExpr(
//                        array.getItems().size() == 1
//                                ? array.getItems().get(0)
//                                : array
//                    , inQuery
//        );
//        replaceInParent(x, inSubQueryExpr);

        return true;
    }
}
