package com.alibaba.fastsql.sql.optimizer.rules;

import com.alibaba.fastsql.sql.SQLUtils;
import com.alibaba.fastsql.sql.ast.SQLExpr;
import com.alibaba.fastsql.sql.ast.SQLName;
import com.alibaba.fastsql.sql.ast.expr.*;
import com.alibaba.fastsql.sql.ast.statement.*;
import com.alibaba.fastsql.sql.dialect.oracle.ast.expr.OracleOuterExpr;

import java.util.Arrays;
import java.util.List;

public class ImplicitJoinRewrite extends OptimizerVisitor {
    public boolean visit(SQLSelectQueryBlock x) {
        if (!(x.getFrom() instanceof SQLJoinTableSource)) {
            return super.visit(x);
        }

        SQLExpr where = x.getWhere();
        if (where == null) {
            return super.visit(x);
        }

        SQLJoinTableSource from = (SQLJoinTableSource) x.getFrom();

        List<SQLExpr> conditions = null;
        if (where instanceof SQLBinaryOpExpr
                && ((SQLBinaryOpExpr) where).getOperator() == SQLBinaryOperator.BooleanAnd) {
            conditions = SQLBinaryOpExpr.split((SQLBinaryOpExpr) where);
        } else if (where instanceof SQLBinaryOpExpr
                && ((SQLBinaryOpExpr) where).getOperator() == SQLBinaryOperator.Equality)
        {
            conditions = Arrays.asList(where);
        } else if (where instanceof SQLBinaryOpExprGroup && ((SQLBinaryOpExprGroup) where).getOperator() == SQLBinaryOperator.BooleanAnd) {
            conditions = ((SQLBinaryOpExprGroup) where).getItems();
        }

        if (conditions == null) {
            return super.visit(x);
        }


        for (int i = conditions.size() - 1; i >= 0; i--) {

            SQLExpr item = conditions.get(i);

            if (!(item instanceof SQLBinaryOpExpr)) {
                continue;
            }

            SQLBinaryOpExpr condition = (SQLBinaryOpExpr) item;
            SQLBinaryOperator operator = condition.getOperator();

            if (operator != SQLBinaryOperator.Equality) {
                continue;
            }

            final SQLExpr conditionLeft = condition.getLeft();
            if (conditionLeft instanceof SQLName
                    && condition.getRight() instanceof SQLName) {
                SQLName left = (SQLName) conditionLeft;
                SQLName right = (SQLName) condition.getRight();

                SQLJoinTableSource join = from.findTableSourceWithColumn(left, right);
                if (join == null) {
                    continue;
                }

                SQLJoinTableSource.JoinType joinType = join.getJoinType();
                if (joinType == SQLJoinTableSource.JoinType.COMMA
                        || joinType == SQLJoinTableSource.JoinType.INNER_JOIN
                        || joinType == SQLJoinTableSource.JoinType.CROSS_JOIN) {
                    SQLExpr joinCondition = condition.clone();

                    if (joinType == SQLJoinTableSource.JoinType.COMMA
                            && join.getLeft() instanceof SQLExprTableSource
                            && join.getRight() instanceof SQLExprTableSource) {
                        final SQLColumnDefinition leftColumn = left.getResolvedColumn();
                        final SQLColumnDefinition rightColumn = right.getResolvedColumn();
                        if (leftColumn != null && leftColumn.isOnlyPrimaryKey()
                                && rightColumn != null && rightColumn.isOnlyPrimaryKey()) {
                            if (SQLUtils.replaceInParent(item, null)) {
                                join.setJoinType(SQLJoinTableSource.JoinType.INNER_JOIN);
                                join.addCondition(joinCondition);
                            }
                        }
                        continue;
                    }
                }
            }

            if (conditionLeft instanceof OracleOuterExpr
                    && ((OracleOuterExpr) conditionLeft).getExpr() instanceof SQLName
                    && condition.getRight() instanceof SQLName) {
                SQLName left = (SQLName) ((OracleOuterExpr) conditionLeft).getExpr();
                SQLName right = (SQLName) condition.getRight();

                SQLJoinTableSource join = from.findTableSourceWithColumn(left, right);
                if (join == null) {
                    continue;
                }

                SQLJoinTableSource.JoinType joinType = join.getJoinType();
                if (joinType == SQLJoinTableSource.JoinType.COMMA
                        || joinType == SQLJoinTableSource.JoinType.RIGHT_OUTER_JOIN) {
                    SQLExpr joinCondition = new SQLBinaryOpExpr(left, operator, right, dbType);

                    if (SQLUtils.replaceInParent(item, null)) {
                        if (joinType == SQLJoinTableSource.JoinType.COMMA) {
                            join.setJoinType(SQLJoinTableSource.JoinType.RIGHT_OUTER_JOIN);
                        }
                        join.addCondition(joinCondition);
                        continue;
                    }
                }
            }
        }

        return super.visit(x);
    }
}
