package com.alibaba.fastsql.sql.optimizer.rules;

import com.alibaba.fastsql.sql.ast.SQLExpr;
import com.alibaba.fastsql.sql.ast.SQLObject;
import com.alibaba.fastsql.sql.ast.expr.SQLBinaryOpExpr;
import com.alibaba.fastsql.sql.ast.expr.SQLBinaryOpExprGroup;
import com.alibaba.fastsql.sql.ast.expr.SQLBinaryOperator;
import com.alibaba.fastsql.sql.ast.statement.SQLSelectQuery;
import com.alibaba.fastsql.sql.ast.statement.SQLSelectQueryBlock;
import com.alibaba.fastsql.sql.ast.statement.SQLSubqueryTableSource;
import com.alibaba.fastsql.sql.ast.statement.SQLUnionQuery;
import com.alibaba.fastsql.sql.ast.statement.SQLUnionQueryTableSource;

import java.util.ArrayList;
import java.util.List;

public class MergeUnion extends OptimizerVisitor {
    public boolean visit(SQLUnionQuery x) {
        switch (x.getOperator()) {
            case UNION:
                break;
            default:
                return true;
        }

        if (x.getRelations().size() > 2) {
            for (SQLSelectQuery relation : x.getRelations()) {
                relation.accept(this);
            }

            SQLSelectQueryBlock first;
            if (x.getRelations().get(0) instanceof SQLSelectQueryBlock) {
                first = (SQLSelectQueryBlock) x.getRelations().get(0).clone();
            } else {
                return false;
            }

            List<Integer> indics = new ArrayList<Integer>();
            for (int i = 1; i < x.getRelations().size(); ++i) {
                SQLSelectQuery item = x.getRelations().get(i);
                if (item instanceof SQLSelectQueryBlock) {
                    SQLSelectQueryBlock right = (SQLSelectQueryBlock) item;

                    if (!first.equalsForMergeJoin(right)) {
                        continue;
                    }

                    if (right.getWhere() != null) {
                        SQLExpr rightWhere = right.getWhere().clone();
                        if (rightWhere instanceof SQLBinaryOpExpr && ((SQLBinaryOpExpr) rightWhere).getOperator() == SQLBinaryOperator.BooleanAnd) {
                            ((SQLBinaryOpExpr) rightWhere).setBracket(true);
                        }

                        first.whereOr(rightWhere);

                        if (first.getWhere() instanceof SQLBinaryOpExprGroup) {
                            ((SQLBinaryOpExprGroup) first.getWhere()).optimize();
                        }
                        indics.add(i);
                    }
                }
            }

            if (indics.size() > 0) {
                for (int i = indics.size() - 1; i >= 0; i--) {
                    x.getRelations().remove(indics.get(i).intValue());
                }
                x.getRelations().set(0, first);
                first.setParent(x);
            }

            return false;
        }

        x.getLeft().accept(this);
        x.getRight().accept(this);

        if (x.getLeft() instanceof SQLSelectQueryBlock && x.getRight() instanceof SQLSelectQueryBlock) {
            SQLSelectQueryBlock left = (SQLSelectQueryBlock) x.getLeft();
            SQLSelectQueryBlock right = (SQLSelectQueryBlock) x.getRight();

            if (!left.equalsForMergeJoin(right)) {
                return false;
            }

            SQLSelectQueryBlock queryBlock = left.clone();
            if (right.getWhere() != null) {
                SQLExpr rightWhere = right.getWhere().clone();
                if (rightWhere instanceof SQLBinaryOpExpr && ((SQLBinaryOpExpr) rightWhere).getOperator() == SQLBinaryOperator.BooleanAnd) {
                    ((SQLBinaryOpExpr) rightWhere).setBracket(true);
                }

                queryBlock.whereOr(rightWhere);

                if (queryBlock.getWhere() instanceof SQLBinaryOpExprGroup) {
                    ((SQLBinaryOpExprGroup) queryBlock.getWhere()).optimize();
                }
            }

            SQLObject parent = x.getParent();
            if (parent instanceof SQLUnionQuery) {
                SQLUnionQuery union = (SQLUnionQuery) parent;
                if (union.getLeft() == x) {
                    union.setLeft(queryBlock);
                } else {
                    union.setRight(queryBlock);
                }
                return false;
            } else if (parent instanceof SQLUnionQueryTableSource) {
                if (parent.getParent() instanceof SQLSelectQueryBlock) {
                    SQLSelectQueryBlock parentParent = (SQLSelectQueryBlock) parent.getParent();
                    if (parentParent.getFrom() == parent) {
                        parentParent.setFrom(new SQLSubqueryTableSource(queryBlock));
                        return false;
                    }
                }
            }
        }

        return false;
    }
}
