package com.alibaba.fastsql.sql.optimizer.rules;

import com.alibaba.fastsql.sql.ast.SQLExpr;
import com.alibaba.fastsql.sql.ast.SQLOrderBy;
import com.alibaba.fastsql.sql.ast.expr.*;
import com.alibaba.fastsql.sql.ast.statement.*;

import java.util.List;

public class Ordinal extends OptimizerVisitor {

    public boolean visit(SQLSelectQueryBlock x) {
        List<SQLSelectItem> selectItems = x.getSelectList();

        for (SQLSelectItem item : selectItems) {
            SQLExpr expr = item.getExpr();
            if (expr instanceof SQLAllColumnExpr) {
                return true;
            }

            if (expr instanceof SQLPropertyExpr && ((SQLPropertyExpr) expr).getName().equals("*")) {
                return true;
            }
        }

        SQLSelectGroupByClause groupBy = x.getGroupBy();

        SQLTableSource from = x.getFrom();

        if (groupBy != null && !groupBy.isWithCube() && !groupBy.isWithRollUp()) {
            for (int i = 0; i < groupBy.getItems().size(); i++) {
                SQLExpr expr = groupBy.getItems().get(i);

                if (expr instanceof SQLIntegerExpr) {
                    continue;
                }

                for (int j = 0; j < selectItems.size(); j++) {
                    SQLSelectItem selectItem = selectItems.get(j);
                    SQLExpr selectItemExpr = selectItem.getExpr();
                    String alias = selectItem.getAlias();
                    if (expr.equals(selectItemExpr)) {
                        SQLExpr indexExpr = new SQLIntegerExpr(j + 1);
                        indexExpr.setParent(x);
                        groupBy.getItems().set(i, indexExpr);
                        optimizedCount++;
                        break;
                    } else if (alias != null
                            && expr instanceof SQLIdentifierExpr
                            && !(selectItem.getExpr() instanceof SQLAggregateExpr)
                            && selectItem.alias_hash() == ((SQLIdentifierExpr) expr).nameHashCode64()) {
                        SQLExpr indexExpr = new SQLIntegerExpr(j + 1);
                        indexExpr.setParent(x);
                        groupBy.getItems().set(i, indexExpr);
                        optimizedCount++;
                    }
                }
            }
        }

        SQLOrderBy orderBy = x.getOrderBy();
        if (orderBy != null && (groupBy == null || (!groupBy.isWithCube() && !groupBy.isWithRollUp()))) {
            final List<SQLSelectOrderByItem> orderByItems = orderBy.getItems();
            for (int i = 0; i < orderByItems.size(); i++) {
                SQLSelectOrderByItem orderByItem = orderByItems.get(i);
                SQLExpr expr = orderByItem.getExpr();
                if (expr instanceof SQLLiteralExpr) {
                    continue;
                }

                for (int j = 0; j < selectItems.size(); j++) {
                    SQLSelectItem selectItem = selectItems.get(j);
                    SQLExpr selectItemExpr = selectItem.getExpr();
                    String alias = selectItem.getAlias();
                    if (expr.equals(selectItemExpr)) {
                        orderByItem.setExpr(new SQLIntegerExpr(j + 1));
                        optimizedCount++;
                        break;
                    } else if (alias != null
                            && expr instanceof SQLIdentifierExpr
                            && selectItem.alias_hash() == ((SQLIdentifierExpr) expr).nameHashCode64()) {
                        orderByItem.setExpr(new SQLIntegerExpr(j + 1));
                        optimizedCount++;
                        break;
                    }
                }
            }

            for (int i = orderByItems.size() - 1; i >= 0; i--) {
                SQLSelectOrderByItem orderByItem = orderByItems.get(i);
                int index = orderByItems.indexOf(orderByItem);
                if (index != i) {
                    orderByItems.remove(i);
                }
            }
        }

        return true;
    }

    public boolean visit(SQLInsertStatement x) {
        return false;
    }

    public boolean visit(SQLUpdateStatement x) {
        return false;
    }

    public boolean visit(SQLDeleteStatement x) {
        return false;
    }

}
