package com.alibaba.fastsql.sql.optimizer.rules;

import com.alibaba.fastsql.sql.SQLUtils;
import com.alibaba.fastsql.sql.ast.*;
import com.alibaba.fastsql.sql.ast.expr.*;
import com.alibaba.fastsql.sql.ast.statement.*;
import com.alibaba.fastsql.sql.dialect.mysql.ast.expr.MySqlOrderingExpr;
import com.alibaba.fastsql.sql.visitor.SQLASTVisitorAdapter;
import com.alibaba.fastsql.sql.visitor.SQLSubQueryGroupVisitor;
import com.alibaba.fastsql.sql.visitor.SQLTableAliasCollectVisitor;
import com.alibaba.fastsql.util.FnvHash;

import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;

public class PushDown extends OptimizerVisitor {
    public boolean visit(SQLSelect x) {
        visitTPCDSQ58(x);
        dupSubQueryToCTE(x);
        visitYouku01(x);

        int optimizeCount = this.optimizedCount;

        final SQLWithSubqueryClause withSubQuery = x.getWithSubQuery();
        if (withSubQuery != null) {
            withSubQuery.accept(this);
        }

        final SQLSelectQuery query = x.getQuery();
        if (query != null) {
            query.accept(this);
        }

        final SQLOrderBy orderBy = x.getOrderBy();
        if (orderBy != null) {
            orderBy.accept(this);
        }


        final SQLExpr offset = x.getOffset();
        if (offset != null) {
            offset.accept(this);
        }

        final SQLExpr rowCount = x.getRowCount();
        if (rowCount != null) {
            rowCount.accept(this);
        }

        if (optimizeCount != this.optimizedCount && x.getParent() instanceof SQLStatement) {
            dupSubQueryToCTE(x);
        }

        return false;
    }

    public boolean visit(SQLSelectQueryBlock x) {
        List<SQLSelectItem> selectList = x.getSelectList();
        for (SQLSelectItem selectItem : selectList) {
            selectItem.accept(this);
        }

        {
            SQLExpr where = x.getWhere();
            if (where != null) {
                where.accept(this);
            }
        }

        {
            SQLTableSource from = x.getFrom();
            if (from != null) {
                from.accept(this);
            }
        }

        visitTPCDS88(x);
        visitTPCDSQ34(x);
        visitTPCDSQ44(x);

        visitTPCDSQ74(x);
        visitTPCDSQ76(x);

        SQLTableSource from = x.getFrom();
        if (!(from instanceof SQLSubqueryTableSource
                && ((SQLSubqueryTableSource) from).getSelect().getQuery() instanceof SQLSelectQueryBlock)) {
            return false;
        }

        SQLSelectQueryBlock subQuery = ((SQLSubqueryTableSource) from).getSelect().getQueryBlock();
        for (SQLSelectItem selectItem : subQuery.getSelectList()) {
            // window function
            SQLExpr expr = selectItem.getExpr();
            if (expr instanceof SQLAggregateExpr) {
                if (((SQLAggregateExpr) expr).getOver() != null) {
                    return false;
                }
            }
        }

        boolean allColumnMatched = false;
        if (selectList.size() == 1) {
            SQLExpr expr = selectList.get(0).getExpr();
            if (expr instanceof SQLAllColumnExpr) {
                allColumnMatched = true;
            } else if (expr instanceof SQLPropertyExpr && ((SQLPropertyExpr) expr).getName().equals("*")) {
                allColumnMatched = true;
            } else if (expr instanceof SQLAggregateExpr) {
                SQLAggregateExpr aggExpr = ((SQLAggregateExpr) expr).clone();
                if (aggExpr.getArguments().size() == 1
                        && aggExpr.methodNameHashCode64() == FnvHash.Constants.COUNT) {
                    SQLExpr arg = aggExpr.getArguments().get(0);
                    ExprReplaceVisitor replaceVisitor = new ExprReplaceVisitor(x, subQuery);
                    arg.accept(replaceVisitor);

                    if (replaceVisitor.repalcedFailCount > 0) {
                        return false;
                    }

                }
            }
        }

        boolean aggregateMatch = false;
        if (!allColumnMatched) {
            List<SQLSelectItem> subSelectList = subQuery.getSelectList();
            if (subSelectList.size() >= selectList.size()) {
                boolean match = true;
                for (int i = 0; i < selectList.size(); i++) {
                    SQLExpr selectItemExpr = selectList.get(i).getExpr();
                    SQLSelectItem subSelectItem = subSelectList.get(i);

                    if (selectItemExpr instanceof SQLIdentifierExpr || selectItemExpr instanceof SQLPropertyExpr) {
                        long nameHashCode64 = ((SQLName) selectItemExpr).nameHashCode64();
                        long subSelectItemHashCode = FnvHash.hashCode64(subSelectItem.computeAlias());
                        if (nameHashCode64 != subSelectItemHashCode) {
                            match = false;
                            break;
                        }
                    } else if (selectItemExpr instanceof SQLAggregateExpr
                            && subSelectList.size() == 1
                            && ((SQLAggregateExpr) selectItemExpr).methodNameHashCode64() == FnvHash.Constants.COUNT
                            && ((SQLAggregateExpr) selectItemExpr).getArguments().size() == 1) {
                        SQLExpr arg = ((SQLAggregateExpr) selectItemExpr).getArguments().get(0);
                        if (arg instanceof SQLIdentifierExpr || arg instanceof SQLPropertyExpr) {
                            long nameHashCode64 = ((SQLName) arg).nameHashCode64();
                            long subSelectItemHashCode = FnvHash.hashCode64(subSelectItem.computeAlias());
                            if (nameHashCode64 != subSelectItemHashCode) {
                                match = false;
                                break;
                            }
                            aggregateMatch = true;
                        }
                    } else {
                        match = false;
                        break;
                    }

                }
                if (match) {
                    allColumnMatched = true;
                }
            }
        }

        SQLExpr where = null;
        if (x.getWhere() != null) {
            where = x.getWhere().clone();
            ExprReplaceVisitor whereVisitor = new ExprReplaceVisitor(x, subQuery);
            where.accept(whereVisitor);

            if (whereVisitor.repalcedFailCount > 0) {
                return false;
            }
        }

        if (allColumnMatched) {
            SQLSelectQueryBlock queryBlock = subQuery.clone();

            if (aggregateMatch) {
                SQLSelectItem subSelectItem = queryBlock.getSelectItem(0);
                SQLSelectItem selectItem = selectList.get(0);

                SQLExpr subSelectItemExpr = subSelectItem.getExpr().clone();
                SQLAggregateExpr aggregateExpr = (SQLAggregateExpr) selectItem.getExpr().clone();
                subSelectItemExpr.setParent(aggregateExpr);
                aggregateExpr.getArguments().set(0, subSelectItemExpr);
                subSelectItem.setExpr(aggregateExpr);
                if (selectItem.getAlias() != null) {
                    subSelectItem.setAlias(selectItem.getAlias());
                }

                if (queryBlock.getDistionOption() == SQLSetQuantifier.DISTINCT) {
                    if (aggregateExpr.getOption() == SQLAggregateOption.DISTINCT) {
                        queryBlock.setDistionOption(0);
                    }
                }
            }

            SQLOrderBy orderBy = x.getOrderBy();
            if (orderBy != null) {
                orderBy = orderBy.clone();
                for (SQLSelectOrderByItem orderByItem : orderBy.getItems()) {
                    SQLExpr expr = orderByItem.getExpr();

                    if (expr instanceof SQLIntegerExpr) {
                        int index = ((SQLIntegerExpr) expr).getNumber().intValue();
                        if (index >= 0 && index < x.getSelectList().size()) {
                            expr = x.getSelectList().get(index).getExpr();
                        }
                    }

                    SQLSelectItem subSelectItem = null;
                    if (expr instanceof SQLPropertyExpr) {
                        SQLPropertyExpr propertyExpr = (SQLPropertyExpr) expr;
                        if (propertyExpr.getOwner() instanceof SQLIdentifierExpr
                                && ((SQLIdentifierExpr) propertyExpr.getOwner()).nameHashCode64() == from.aliasHashCode64()) {
                            long nameHash = propertyExpr.nameHashCode64();
                            subSelectItem = queryBlock.findSelectItem(nameHash);
                        }
                    } else if (expr instanceof SQLIdentifierExpr) {
                        long nameHash = ((SQLIdentifierExpr) expr).nameHashCode64();
                        subSelectItem = queryBlock.findSelectItem(nameHash);
                    }

                    if (subSelectItem == null) {
                        return false; // can not merge
                    }

                    orderByItem.setExpr(subSelectItem.getExpr().clone());

                    if (!queryBlock.containsOrderBy(orderByItem)) {
                        queryBlock.addOrderBy(orderByItem);
                    }
                }
            }

            final AtomicBoolean aggregate = new AtomicBoolean();
            if (queryBlock.getGroupBy() != null) {
                if (where != null && x.getGroupBy() != null) {
                    return false;
                }
                aggregate.set(true);
            }

            if (aggregate.get() && where != null) {
                SQLASTVisitorAdapter v = new SQLASTVisitorAdapter() {
                    public boolean visit(SQLAggregateExpr x) {
                        aggregate.set(true);
                        return false;
                    }
                };
                where.accept(v);
            }

            if (aggregate.get()) {
                queryBlock.addHaving(where);
            } else {
                queryBlock.addWhere(where);
            }
//            queryBlock.addWhere(where);

            SQLLimit limit = x.getLimit();
            if (limit != null) {
                if (queryBlock.getLimit() == null) {
                    queryBlock.setLimit(limit);
                } else {
                    return false; // TODO limit merge
                }
            }

            SQLObject parent = x.getParent();

            if (parent instanceof SQLSelect) {
                SQLSelect select = (SQLSelect) x.getParent();
                select.setQuery(queryBlock);
                queryBlock.accept(this);

            } else if (parent instanceof SQLUnionQuery) {
                SQLUnionQuery union = (SQLUnionQuery) x.getParent();

                if (union.getLeft() == x) {
                    union.setLeft(queryBlock);
                } else {
                    union.setRight(queryBlock);
                }

                queryBlock.accept(this);
            }
        }

        return false;
    }

    class ExprReplaceVisitor extends SQLASTVisitorAdapter {
        private final SQLSelectQueryBlock queryBlock;
        private final SQLSelectQueryBlock subQueryBlock;

        private int repalcedFailCount = 0;

        public ExprReplaceVisitor(SQLSelectQueryBlock queryBlock, SQLSelectQueryBlock subQueryBlock) {
            this.queryBlock = queryBlock;
            this.subQueryBlock = subQueryBlock;
        }

        public boolean visit(SQLIdentifierExpr x) {
            SQLSelectItem selectItem = subQueryBlock.findSelectItem(x.nameHashCode64());
            if (selectItem == null) {
                repalcedFailCount++;
                return false;
            }

            if (!SQLUtils.replaceInParent(x, selectItem.getExpr().clone())) {
                repalcedFailCount++;
            }

            return false;
        }

        public boolean visit(SQLPropertyExpr x) {
            if (!(x.getOwner() instanceof SQLIdentifierExpr)) {
                repalcedFailCount++;
                return false;
            }

            SQLIdentifierExpr owner = (SQLIdentifierExpr) x.getOwner();
            if (queryBlock.getFrom().aliasHashCode64() != owner.nameHashCode64()) {
                repalcedFailCount++;
                return false;
            }

            if (x.getName().equals("*")) {
                if (!SQLUtils.replaceInParent(x, new SQLAllColumnExpr())) {
                    repalcedFailCount++;
                }
                return false;
            }

            SQLSelectItem selectItem = subQueryBlock.findSelectItem(x.nameHashCode64());
            if (selectItem == null) {
                repalcedFailCount++;
                return false;
            }

            if (!SQLUtils.replaceInParent(x, selectItem.getExpr().clone())) {
                repalcedFailCount++;
            }

            return false;
        }
    }

    public void visitTPCDS88(SQLSelectQueryBlock x) {
        final SQLExpr where = x.getWhere();
        if (where == null) {
            return;
        }

        final List<SQLExpr> conditions = SQLBinaryOpExpr.split(where, SQLBinaryOperator.BooleanAnd);
        if (conditions.size() < 2) {
            return;
        }

        Map<SQLName, List<SQLBinaryOpExpr>> lsNameLiteralsMap = new LinkedHashMap<SQLName, List<SQLBinaryOpExpr>>();

        List<SQLBinaryOpExpr> l2NameLiterals = new ArrayList<SQLBinaryOpExpr>();

        Map<Long, SQLName> l1Names = new LinkedHashMap<Long, SQLName>();

        int orCount = 0, orItemCount = 0;
        for (SQLExpr condition : conditions) {
            if (ConstFolding.isLeftNameAndRightLiteral(condition)) {
                SQLBinaryOpExpr nameLiteral = (SQLBinaryOpExpr) condition;
                final SQLName name = (SQLName) nameLiteral.getLeft();
                l1Names.put((name).nameHashCode64(), name);
            } else if (ConstFolding.isBothName(condition)) {
                SQLBinaryOpExpr nameName = (SQLBinaryOpExpr) condition;
                final SQLName leftNamename = (SQLName) nameName.getLeft();
                final SQLName rightRame = (SQLName) nameName.getRight();
                l1Names.put((leftNamename).nameHashCode64(), leftNamename);
                l1Names.put((rightRame).nameHashCode64(), rightRame);

            } else if (condition instanceof SQLBinaryOpExpr
                    && ((SQLBinaryOpExpr) condition).getOperator() == SQLBinaryOperator.BooleanOr) {
                return; // TODO
            } else if (condition instanceof SQLBinaryOpExprGroup
                    && ((SQLBinaryOpExprGroup) condition).getOperator() == SQLBinaryOperator.BooleanOr) {
                orCount ++;
                SQLBinaryOpExprGroup orCondition = (SQLBinaryOpExprGroup) condition;

                for (SQLExpr item : orCondition.getItems()) {
                    if (item instanceof SQLBinaryOpExpr
                            && ((SQLBinaryOpExpr) item).getOperator() == SQLBinaryOperator.BooleanAnd) {
                        List<SQLExpr> itemItems = SQLBinaryOpExpr.split(item, SQLBinaryOperator.BooleanAnd);
                        orItemCount++;

                        for (SQLExpr itemItem : itemItems) {
                            if (ConstFolding.isLeftNameAndRightLiteral(itemItem)) {
                                l2NameLiterals.add((SQLBinaryOpExpr) itemItem);
                            } else {
                                return;
                            }
                        }
                    } else {
                        return;
                    }
                }
            } else {
                return;
            }
        }

        if (orCount != 1) {
            return;
        }

        for (int i = l2NameLiterals.size() - 1; i >= 0; i--) {
            SQLBinaryOpExpr nameLiteral = l2NameLiterals.get(i);
            final SQLExpr left = nameLiteral.getLeft();
            List<SQLBinaryOpExpr> literals = lsNameLiteralsMap.get(left);
            if (literals == null) {
                literals = new ArrayList<SQLBinaryOpExpr>();
                lsNameLiteralsMap.put((SQLName) left, literals);
            }
            literals.add(nameLiteral);
        }

        if (lsNameLiteralsMap.size() == 0) {
            return;
        }

        SQLTableSource resolvedTableSource = null;
        for (Map.Entry<SQLName, List<SQLBinaryOpExpr>> entry : lsNameLiteralsMap.entrySet()) {
            final SQLName name = entry.getKey();

            if (l1Names.containsKey(name.nameHashCode64())) {
                return;
            }

            List<SQLBinaryOpExpr> nameLiterals = entry.getValue();
            if (nameLiterals.size() != orItemCount) {
                return;
            }

            if (name instanceof SQLPropertyExpr) {
                SQLPropertyExpr propertyExpr = (SQLPropertyExpr) name;
                if (resolvedTableSource == null) {
                    resolvedTableSource = propertyExpr.getResolvedTableSource();
                } else if (resolvedTableSource != propertyExpr.getResolvedTableSource()) {
                    return;
                }
            }
        }

        if (resolvedTableSource == null) {
            return;
        }

        for (Map.Entry<SQLName, List<SQLBinaryOpExpr>> entry : lsNameLiteralsMap.entrySet()) {
            final SQLName name = entry.getKey();
            List<SQLBinaryOpExpr> nameLiterals = entry.getValue();

            boolean allEq = true;
            for (SQLBinaryOpExpr nameLiteral : nameLiterals) {
                if (nameLiteral.getOperator() != SQLBinaryOperator.Equality) {
                    allEq = false;
                    break;
                }
            }

            SQLExpr pushDownCondition = null;
            if (allEq) {
                SQLInListExpr in = new SQLInListExpr(name.clone());
                for (SQLBinaryOpExpr nameLiteral : nameLiterals) {
                    in.addTarget(nameLiteral.getRight().clone());
                }
                pushDownCondition = in;

            } else {
                SQLBinaryOpExprGroup group = new SQLBinaryOpExprGroup(SQLBinaryOperator.BooleanOr);
                for (SQLBinaryOpExpr nameLiteral : nameLiterals) {
                    group.add(nameLiteral.clone());
                }
                new ConstFolding()
                        .visit(group);
                pushDownCondition = group;
            }

            if (resolvedTableSource instanceof SQLExprTableSource) {
                resolvedTableSource = pushDownNameLiterals((SQLExprTableSource) resolvedTableSource, pushDownCondition);
            } else if (resolvedTableSource instanceof SQLSubqueryTableSource) {
                pushDownNameLiterals((SQLSubqueryTableSource) resolvedTableSource, pushDownCondition);
            }
        }


    }

    public void visitTPCDSQ76(SQLSelectQueryBlock x) {
        {
            final SQLSelectGroupByClause groupBy = x.getGroupBy();
            if (groupBy == null
                    || groupBy.getItems().size() == 0
                    || groupBy.getHaving() != null) {
                return;
            }

            for (SQLExpr item : groupBy.getItems()) {
                if (!(item instanceof SQLIdentifierExpr)) {
                    return;
                }
            }
        }

        for (SQLSelectItem selectItem : x.getSelectList()) {
            final SQLExpr expr = selectItem.getExpr();
            if (expr instanceof SQLIdentifierExpr) {
                continue;
            } else if (expr instanceof SQLAggregateExpr) {
                if (selectItem.getAlias() == null) {
                    return;
                }

                SQLAggregateExpr aggExpr = (SQLAggregateExpr) expr;
                if (aggExpr.getOver() != null || aggExpr.getOverRef() != null || aggExpr.getArguments().size() != 1) {
                    return;
                }

                if (aggExpr.methodNameHashCode64() != FnvHash.Constants.COUNT
                        && aggExpr.methodNameHashCode64() != FnvHash.Constants.SUM
                        && aggExpr.methodNameHashCode64() != FnvHash.Constants.MAX
                        && aggExpr.methodNameHashCode64() != FnvHash.Constants.MIN) {
                    return;
                }

                SQLExpr arg = aggExpr.getArguments().get(0);
                if (arg instanceof SQLAllColumnExpr || arg instanceof SQLIdentifierExpr) {

                } else {
                    return;
                }
            } else {
                return;
            }
        }

        if (!(x.getFrom() instanceof SQLUnionQueryTableSource)) {
            return;
        }

        SQLSelectQueryBlock x1 = x.clone();
        SQLUnionQuery union = ((SQLUnionQueryTableSource) x1.getFrom()).getUnion();
        List<SQLSelectQuery> queries = new ArrayList<SQLSelectQuery>();
        final SQLSelectGroupByClause groupBy = x1.getGroupBy();
        {
            SQLUnionOperator operator = union.getOperator();
            SQLSelectQuery left = union.getLeft();
            SQLSelectQuery right = union.getRight();

            if (left instanceof SQLUnionQuery && ((SQLUnionQuery) left).getOperator() == operator) {
                SQLUnionQuery leftUnion = (SQLUnionQuery) left;

                queries.add(right);

                for (; ; ) {
                    SQLSelectQuery leftLeft = leftUnion.getLeft();
                    SQLSelectQuery leftRight = leftUnion.getRight();

                    if ((!leftUnion.isBracket())
                            && leftUnion.getOrderBy() == null
                            && (!leftLeft.isBracket())
                            && (!leftRight.isBracket())
                            && leftLeft instanceof SQLUnionQuery
                            && ((SQLUnionQuery) leftLeft).getOperator() == operator) {
                        queries.add(leftRight);
                        leftUnion = (SQLUnionQuery) leftLeft;
                        continue;
                    } else {
                        queries.add(leftRight);
                        queries.add(leftLeft);
                    }
                    break;
                }

                for (int i = queries.size() - 1; i >= 0; i--) {
                    SQLSelectQuery item = queries.get(i);
                    item.accept(this);
                }
            }
        }

        for (SQLSelectQuery query : queries) {
            if (!(query instanceof SQLSelectQueryBlock)) {
                return;
            }

            SQLSelectQueryBlock queryBlock = (SQLSelectQueryBlock) query;
            if (queryBlock.getGroupBy() != null) {
                return;
            }
        }

        Set<Object> literalValues = new LinkedHashSet<Object>();
        int firstLiteralItemCount = 0;
        if (queries.size() > 0) {
            SQLSelectQueryBlock queryBlock = (SQLSelectQueryBlock) queries.get(0);
            for (int i = 0; i < queryBlock.getSelectList().size(); i++) {
                SQLExpr expr = queryBlock.getSelectList().get(i).getExpr();
                if (expr instanceof SQLValuableExpr) {
                    firstLiteralItemCount = i;
                    if (i == 0) {
                        literalValues.add(((SQLValuableExpr) expr).getValue());
                    }
                } else {
                    break;
                }
            }
        }
        if (firstLiteralItemCount == 0) {
            return;
        }

        for (int i = 1; i < queries.size(); i++) {
            int literalItemCount = 0;
            SQLSelectQueryBlock queryBlock = (SQLSelectQueryBlock) queries.get(i);
            for (int j = 0; j < queryBlock.getSelectList().size(); j++) {
                SQLExpr expr = queryBlock.getSelectList().get(j).getExpr();
                if (expr instanceof SQLValuableExpr) {
                    literalItemCount = j;
                    if (j == 0) {
                        literalValues.add(((SQLValuableExpr) expr).getValue());
                    }
                } else {
                    break;
                }
            }
            if (literalItemCount < firstLiteralItemCount) {
                return;
            }
        }

        if (literalValues.size() != queries.size()) {
            return;
        }

        for (int i = 0; i < queries.size(); i++) {
            SQLSelectQueryBlock queryBlock = (SQLSelectQueryBlock) queries.get(i);
            SQLSelectGroupByClause subGroupBy = groupBy.clone();
            final List<SQLExpr> subGroupByItems = subGroupBy.getItems();
            for (int j = subGroupByItems.size() - 1; j >= 0; j--) {
                SQLExpr item = subGroupByItems.get(j);
                SQLIdentifierExpr identItem = (SQLIdentifierExpr) item;
                SQLSelectItem selectItem = queryBlock.findSelectItem(identItem.hashCode64());
                if (selectItem == null) {
                    return;
                }
                final SQLExpr selectItemExpr = selectItem.getExpr();
                if (selectItemExpr instanceof SQLLiteralExpr) {
                    subGroupByItems.remove(i);
                } else if (selectItemExpr.equals(item)) {
                    // continue
                } else {
                    SQLExpr selectItemExpr2 = selectItemExpr.clone();
                    selectItemExpr2.setParent(subGroupBy);
                    subGroupByItems.set(i, selectItemExpr2);
                }
            }
            queryBlock.setGroupBy(subGroupBy);

            List<SQLSelectItem> newSelectItems = new ArrayList<SQLSelectItem>(x.getSelectList().size());
            for (SQLSelectItem selectItem : x.getSelectList()) {
                SQLSelectItem newSelectItem;
                final SQLExpr selectItemExpr = selectItem.getExpr();
                if (selectItemExpr instanceof SQLIdentifierExpr) {
                    SQLSelectItem subSelectItem = queryBlock.findSelectItem(((SQLIdentifierExpr) selectItemExpr).hashCode64());
                    if (subSelectItem == null) {
                        return;
                    }
                    newSelectItem = subSelectItem.clone();
                } else if (selectItemExpr instanceof SQLAggregateExpr) {
                    SQLExpr aggArg = ((SQLAggregateExpr) selectItemExpr).getArguments().get(0);
                    if (aggArg instanceof SQLAllColumnExpr) {
                        newSelectItem = selectItem.clone();
                    } else if (aggArg instanceof SQLIdentifierExpr) {
                        newSelectItem = selectItem.clone();

                        SQLSelectItem subSelectItem = queryBlock.findSelectItem(((SQLIdentifierExpr) aggArg).hashCode64());
                        if (subSelectItem.getExpr().equals(aggArg)) {

                        } else if (subSelectItem.getExpr() instanceof SQLName) {
                            SQLExpr arg2 = subSelectItem.getExpr().clone();
                            final SQLAggregateExpr agg = (SQLAggregateExpr) newSelectItem.getExpr();
                            agg.setArgument(0, arg2);
                        } else {
                            return;
                        }
                    } else {
                        return;
                    }
                } else {
                    return;
                }
                newSelectItem.setParent(queryBlock);
                newSelectItems.add(newSelectItem);
            }
            queryBlock.getSelectList().clear();
            queryBlock.getSelectList().addAll(newSelectItems);
        }

        for (SQLSelectItem selectItem : x1.getSelectList()) {
            SQLExpr selectItemExpr = selectItem.getExpr();
            if (selectItemExpr instanceof SQLAggregateExpr) {
                SQLAggregateExpr aggregateExpr = (SQLAggregateExpr) selectItemExpr;
                if (aggregateExpr.methodNameHashCode64() == FnvHash.Constants.COUNT) {
                    aggregateExpr.setMethodName("SUM");
                }

                SQLIdentifierExpr arg = new SQLIdentifierExpr(selectItem.getAlias());
                aggregateExpr.setArgument(0, arg);
            }
        }

        x.replaceInParent(x1);
    }


    public void visitTPCDSQ34(SQLSelectQueryBlock x) {
        final SQLObject parent = x.getParent();
        if(!(parent instanceof SQLSelect)) {
            return;
        }
        SQLSelect select = (SQLSelect) parent;

        if (!(select.getParent() instanceof SQLSelectStatement)) {
            return;
        }

        final SQLWithSubqueryClause with = select.getWithSubQuery();
        if (with == null || with.getRecursive() != null) {
            return;
        }

        final SQLTableSource from = x.getFrom();
        List<SQLTableSource> outTableSources = new ArrayList<SQLTableSource>();
        {
            if (from instanceof SQLExprTableSource) {
                outTableSources.add(from);
            } else if (from instanceof SQLJoinTableSource) {
                SQLJoinTableSource join = (SQLJoinTableSource) from;
                join.splitTo(outTableSources, join.getJoinType());
            } else {
                return;
            }
        }

        Map<Long, SQLTableSource> tableSourceMap = new HashMap<Long, SQLTableSource>();
        for (SQLTableSource tableSource : outTableSources) {
            if (!(tableSource instanceof SQLExprTableSource)) {
                return;
            }
            SQLExpr expr = ((SQLExprTableSource) tableSource).getExpr();
            if(!(expr instanceof SQLIdentifierExpr)) {
                return;
            }

            if (tableSource.getAlias() == null) {
                tableSourceMap.put(((SQLIdentifierExpr) expr).nameHashCode64(), tableSource);
            } else {
                tableSourceMap.put(tableSource.aliasHashCode64(), tableSource);
            }
        }

        final SQLExpr where = x.getWhere();
        if (where == null) {
            return;
        }

        Map<Long, Map<Long, List<SQLBinaryOpExpr>>> tableNameLiteralMap = new HashMap<Long, Map<Long, List<SQLBinaryOpExpr>>>();

        List<SQLExpr> conditions = SQLBinaryOpExpr.split(where, SQLBinaryOperator.BooleanAnd);
        for (int i = conditions.size() - 1; i >= 0; i--) {
            SQLExpr condition = conditions.get(i);
            if (ConstFolding.isLeftNameAndRightLiteral(condition)) {
                SQLBinaryOpExpr binaryOpExpr = (SQLBinaryOpExpr) condition;
                final SQLExpr left = binaryOpExpr.getLeft();

                SQLTableSource tableSource;
                long ownerHashCode64;
                long hashCode;
                if (left instanceof SQLPropertyExpr) {
                    SQLPropertyExpr name = (SQLPropertyExpr) left;
                    if (!(name.getOwner() instanceof SQLIdentifierExpr)) {
                        return;
                    }
                    SQLIdentifierExpr owner = (SQLIdentifierExpr) name.getOwner();
                    ownerHashCode64 = owner.nameHashCode64();
                    hashCode = name.nameHashCode64();
                    tableSource = tableSourceMap.get(ownerHashCode64);
                } else if (left instanceof SQLIdentifierExpr) {
                    SQLIdentifierExpr name = (SQLIdentifierExpr) left;
                    hashCode = name.nameHashCode64();
                    tableSource = from.findTableSourceWithColumn(name);
                    if (!(tableSource instanceof SQLExprTableSource)) {
                        return;
                    }
                    if (!(((SQLExprTableSource) tableSource).getExpr() instanceof SQLIdentifierExpr)) {
                        return;
                    }
//                    SQLObject object = name.getResolvedOwnerObject();
//                    if (object instanceof SQLWithSubqueryClause.Entry) {
//                        SQLWithSubqueryClause.Entry entry = (SQLWithSubqueryClause.Entry) object;
//                        if (entry.getParent() != with || entry.getAlias() == null) {
//                            return;
//                        }
//                        ownerHashCode64 = entry.aliasHashCode64();
//                    } else {
//                        return;
//                    }
                } else {
                    return;
                }

                if (tableSource == null) {
                    continue;
                }

                SQLIdentifierExpr tableSourceExpr = (SQLIdentifierExpr) ((SQLExprTableSource) tableSource).getExpr();

                Map<Long, List<SQLBinaryOpExpr>> nameLiterals = tableNameLiteralMap.get(tableSourceExpr.nameHashCode64());
                if (nameLiterals == null) {
                    nameLiterals = new HashMap<Long, List<SQLBinaryOpExpr>>();
                    tableNameLiteralMap.put(tableSourceExpr.nameHashCode64(), nameLiterals);
                }




                List<SQLBinaryOpExpr> literals = nameLiterals.get(hashCode);
                if (literals == null) {
                    literals = new ArrayList<SQLBinaryOpExpr>();
                    nameLiterals.put(hashCode, literals);
                }
                literals.add(binaryOpExpr);
                conditions.remove(i);
            }
        }

        for (Map.Entry<Long, Map<Long, List<SQLBinaryOpExpr>>> tableNameLiteralMapEntry : tableNameLiteralMap.entrySet()) {
            long tableNameHashCode = tableNameLiteralMapEntry.getKey();
            Map<Long, List<SQLBinaryOpExpr>> nameLiterals = tableNameLiteralMapEntry.getValue();

            if (nameLiterals.size() == 0) {
                continue;
            }

            int matchCount = 0;
            for (Map.Entry<Long, List<SQLBinaryOpExpr>> entry : nameLiterals.entrySet()) {
                long hashCode64 = entry.getKey();
                MatchName v = new MatchName(hashCode64);
                for (SQLExpr condition : conditions) {
                    condition.accept(v);
                }
                if ((matchCount = v.matchCount) > 0) {
                    continue;
                }
            }
            if (matchCount > 0) {
                continue;
            }

            SQLWithSubqueryClause.Entry entry = with.findEntry(tableNameHashCode);
            if (entry == null) {
                return;
            }
            for (List<SQLBinaryOpExpr> literals : nameLiterals.values()) {
                final SQLBinaryOpExpr firstLiteral = literals.get(0);
                boolean allEq = true;
                for (SQLBinaryOpExpr literal : literals) {
                    if (literal.getOperator() != SQLBinaryOperator.Equality) {
                        allEq = false;
                        break;
                    }
                }

                SQLIdentifierExpr name = new SQLIdentifierExpr(((SQLName) firstLiteral.getLeft()).getSimpleName());

                SQLExpr condition;
                if (literals.size() == 1) {
                    condition = new SQLBinaryOpExpr(name, firstLiteral.getOperator(), firstLiteral.getRight());
                } else {
                    if (allEq) {
                        SQLInListExpr inList = new SQLInListExpr(name);
                        final List<SQLExpr> inListTargetList = inList.getTargetList();
                        for (SQLBinaryOpExpr literal : literals) {
                            final SQLExpr value = literal.getRight().clone();
                            if (!inListTargetList.contains(value)) {
                                inList.addTarget(value);
                            }
                        }
                        if (inListTargetList.size() == 1) {
                            condition = new SQLBinaryOpExpr(inList.getExpr()
                                    , SQLBinaryOperator.Equality
                                    , inListTargetList.get(0));
                        } else {
                            inList.sortTargetList();
                            condition = inList;
                        }
                    } else {
                        continue;
                    }
                }

                pushDownNameLiterals(entry
                        .getSubQuery()
                        .getQuery(), condition);
            }
        }
    }

    private SQLTableSource pushDownNameLiterals(SQLExprTableSource tableSource, SQLExpr condition) {
        final SQLExpr expr = tableSource.getExpr();
        if (!(expr instanceof SQLIdentifierExpr)) {
            return tableSource;
        }

        SQLExprTableSource subTableSource = tableSource.clone();
        subTableSource.setAlias(null);
        SQLIdentifierExpr table = (SQLIdentifierExpr) expr;

        SQLSelectQueryBlock queryBlock = new SQLSelectQueryBlock(dbType);
        queryBlock.addSelectItem(new SQLAllColumnExpr());
        queryBlock.setFrom(subTableSource);
        queryBlock.addWhere(condition);

        String alias = tableSource.getAlias();
        if (alias == null) {
            alias = table.normalizedName();
        }

        List<SQLName> names = new ArrayList<SQLName>();
        if (condition instanceof SQLInListExpr) {
            SQLExpr left = ((SQLInListExpr) condition).getExpr();
            if (left instanceof SQLName) {
                names.add((SQLName) left);
            } else {
                return tableSource;
            }
        } else if (condition instanceof SQLBinaryOpExpr) {
            SQLExpr left = ((SQLBinaryOpExpr) condition).getLeft();
            if (left instanceof SQLName) {
                names.add((SQLName) left);
            } else {
                return tableSource;
            }
        } else if (condition instanceof SQLBinaryOpExprGroup) {
            SQLBinaryOpExprGroup group = (SQLBinaryOpExprGroup) condition;
            for (SQLExpr item : group.getItems()) {
                if (item instanceof SQLBinaryOpExpr) {
                    SQLBinaryOpExpr binaryOpExpr = (SQLBinaryOpExpr) item;
                    if (binaryOpExpr.getLeft() instanceof SQLName) {
                        names.add((SQLName) binaryOpExpr.getLeft());
                    } else {
                        return tableSource;
                    }
                } else {
                    return tableSource;
                }
            }
        } else {
            return tableSource;
        }

        for (SQLName name : names) {
            if (name instanceof SQLPropertyExpr) {
                SQLPropertyExpr propertyExpr = (SQLPropertyExpr) name;
                if (table.equals(propertyExpr.getOwner())) {
                    if (SQLUtils.replaceInParent(name, new SQLIdentifierExpr(propertyExpr.getName()))) {

                    } else {
                        return tableSource;
                    }
                }
            }
        }

        SQLSubqueryTableSource subqueryTableSource = new SQLSubqueryTableSource(queryBlock, alias);
        if (SQLUtils.replaceInParent(tableSource, subqueryTableSource)) {
            optimizedCount++;
            return subqueryTableSource;
        }

        return tableSource;
    }

    private void pushDownNameLiterals(SQLSubqueryTableSource tableSource, SQLExpr condition) {
        String alias = tableSource.getAlias();
        if (alias == null) {
            return;
        }

        List<SQLName> names = new ArrayList<SQLName>();
        if (condition instanceof SQLInListExpr) {
            SQLExpr left = ((SQLInListExpr) condition).getExpr();
            if (left instanceof SQLName) {
                names.add((SQLName) left);
            } else {
                return;
            }
        } else if (condition instanceof SQLBinaryOpExpr) {
            SQLExpr left = ((SQLBinaryOpExpr) condition).getLeft();
            if (left instanceof SQLName) {
                names.add((SQLName) left);
            } else {
                return;
            }
        } else if (condition instanceof SQLBinaryOpExprGroup) {
            SQLBinaryOpExprGroup group = (SQLBinaryOpExprGroup) condition;
            for (SQLExpr item : group.getItems()) {
                if (item instanceof SQLBinaryOpExpr) {
                    SQLBinaryOpExpr binaryOpExpr = (SQLBinaryOpExpr) item;
                    if (binaryOpExpr.getLeft() instanceof SQLName) {
                        names.add((SQLName) binaryOpExpr.getLeft());
                    } else {
                        return;
                    }
                } else {
                    return;
                }
            }
        } else {
            return;
        }

        long aliasHashCode64 = FnvHash.hashCode64(alias);
        for (SQLName name : names) {
            if (name instanceof SQLPropertyExpr) {
                SQLPropertyExpr propertyExpr = (SQLPropertyExpr) name;
                final SQLExpr owner = propertyExpr.getOwner();
                if (owner instanceof SQLIdentifierExpr && ((SQLIdentifierExpr) owner).nameHashCode64() == aliasHashCode64) {
                    if (SQLUtils.replaceInParent(name, new SQLIdentifierExpr(propertyExpr.getName()))) {

                    } else {
                        return;
                    }
                } else {
                    return;
                }
            }
        }

        pushDownNameLiterals(tableSource.getSelect().getQuery(), condition);
    }

    private void pushDownNameLiterals(SQLSelectQuery query, SQLExpr condition) {
        if (query instanceof SQLUnionQuery) {
            SQLUnionQuery union = (SQLUnionQuery) query;
            pushDownNameLiterals(union.getLeft(), condition.clone());
            pushDownNameLiterals(union.getRight(), condition.clone());
            return;
        }

        if (query instanceof  SQLSelectQueryBlock){
            SQLSelectQueryBlock queryBlock = (SQLSelectQueryBlock) query;

            SQLIdentifierExpr name;
            if (condition instanceof SQLInListExpr) {
                name = (SQLIdentifierExpr) ((SQLInListExpr) condition).getExpr();
            } else if (condition instanceof SQLBinaryOpExpr) {
                name = (SQLIdentifierExpr) ((SQLBinaryOpExpr) condition).getLeft();
            } else {
                return;
            }

            final SQLTableSource from = queryBlock.getFrom();
            SQLObject resolveColum = queryBlock.resolveColum(name.hashCode64());

            SQLExpr selectItemExpr;
            if (resolveColum instanceof SQLSelectItem) {
                SQLSelectItem selectItem = (SQLSelectItem) resolveColum;
                selectItemExpr = selectItem.getExpr();
                if (!SQLUtils.replaceInParent(name, selectItemExpr.clone())) {
                    return;
                }

            } else if (resolveColum instanceof SQLColumnDefinition) {
                selectItemExpr = name;
            } else {
                return;
            }

            if (from instanceof SQLSubqueryTableSource) {
                if (selectItemExpr instanceof SQLIdentifierExpr) {
                } else if (selectItemExpr instanceof SQLPropertyExpr) {
                    return; // TODO
                } else {
                    return;
                }
                pushDownNameLiterals(((SQLSubqueryTableSource) from).getSelect().getQuery(), condition);
                return;
            } else if (from instanceof SQLUnionQueryTableSource) {
                if (selectItemExpr instanceof SQLIdentifierExpr) {
                } else if (selectItemExpr instanceof SQLPropertyExpr) {
                    return; // TODO
                } else {
                    return;
                }

                pushDownNameLiterals(((SQLUnionQueryTableSource) from).getUnion(), condition);
                return;
            }

            if (selectItemExpr instanceof SQLIdentifierExpr) {
                SQLObject column = queryBlock.resolveColum(
                        ((SQLIdentifierExpr) selectItemExpr).nameHashCode64()
                );
                if (column != null) {
                    final SQLExpr where = queryBlock.getWhere();
                    if (where != null) {
                        List<SQLExpr> items = SQLBinaryOpExpr.split(where, SQLBinaryOperator.BooleanAnd);
                        if (items.contains(condition)) {
                            return;
                        }
                    }
                    queryBlock.addWhere(condition.clone());
                    optimizedCount++;
                }
            }
        }
    }

    static class MatchName extends SQLASTVisitorAdapter {
        public final long nameHashCode64;
        public int matchCount;

        public MatchName(long nameHashCode64) {
            this.nameHashCode64 = nameHashCode64;
        }

        public boolean visit(SQLIdentifierExpr x) {
            if (x.nameHashCode64() == nameHashCode64) {
                matchCount++;
            }
            return false;
        }

        public boolean visit(SQLPropertyExpr x) {
            if (x.nameHashCode64() == nameHashCode64) {
                matchCount++;
            }
            return true;
        }
    }

    public void visitTPCDSQ58(SQLSelect select) {
        if (!(select.getParent() instanceof SQLSelectStatement)) {
            return;
        }

        final SQLWithSubqueryClause with = select.getWithSubQuery();
        if (with == null) {
            return;
        }

        List<SQLInSubQueryExpr> inSubQueryExprList = new ArrayList<SQLInSubQueryExpr>();
        for (SQLWithSubqueryClause.Entry entry : with.getEntries()) {
            final SQLSelectQueryBlock queryBlock = entry.getSubQuery().getQueryBlock();
            if (queryBlock == null) {
                return;
            }
            final SQLExpr where = queryBlock.getWhere();
            if (where == null) {
                return;
            }

            if (where != null) {
                final List<SQLExpr> list = SQLBinaryOpExpr.split(where, SQLBinaryOperator.BooleanAnd);
                for (SQLExpr item : list) {
                    if (item instanceof SQLInSubQueryExpr) {
                        inSubQueryExprList.add((SQLInSubQueryExpr) item);
                    }
                }
            }

        }

        if (inSubQueryExprList.size() < 2) {
            return;
        }
        final SQLInSubQueryExpr first = inSubQueryExprList.get(0);
        for (int i = 1; i < inSubQueryExprList.size(); i++) {
            if (!first.getSubQuery().equals(inSubQueryExprList.get(0).getSubQuery())) {
                return;
            }
        }

        final SQLSelectQueryBlock firstQueryBlock = first.getSubQuery().getQueryBlock();
        if (firstQueryBlock == null || firstQueryBlock.getSelectList().size() != 1) {
            return;
        }

        SQLTableAliasCollectVisitor v = new SQLTableAliasCollectVisitor();
        select.accept(v);
        String alias = v.genAlias(1);
        if (alias == null) {
            return;
        }

        SQLWithSubqueryClause.Entry entry = new SQLWithSubqueryClause.Entry(alias,
                first.getSubQuery().clone());
        entry.setParent(with);
        with.getEntries().add(0, entry);


        SQLSelectQueryBlock inQueryBlock = new SQLSelectQueryBlock(dbType);
        inQueryBlock.setFrom(new SQLExprTableSource(new SQLIdentifierExpr(alias)));
        inQueryBlock.addSelectItem(firstQueryBlock.getSelectItem(0).clone());

        for (SQLInSubQueryExpr inSubQueryExpr : inSubQueryExprList) {
            inSubQueryExpr.setSubQuery(new SQLSelect(inQueryBlock.clone()));
        }
    }

    public void dupSubQueryToCTE(SQLSelect select) {
        if (!(select.getParent() instanceof SQLSelectStatement)) {
            return;
        }

        SQLWithSubqueryClause with = select.getWithSubQuery();
        if (with != null) {
            return;
        }

        final SQLSelectQueryBlock queryBlock = select.getQueryBlock();
        if (queryBlock == null) {
            return;
        }

        final SQLTableSource from = queryBlock.getFrom();

        if (!(from instanceof SQLJoinTableSource)) {
            return;
        }

        List<List<SQLSubqueryTableSource>> groupedSubqueryTableSources = new ArrayList<List<SQLSubqueryTableSource>>();
        {
            SQLSubQueryGroupVisitor v = new SQLSubQueryGroupVisitor(dbType);
            from.accept(v);


            for (List<SQLSubqueryTableSource> subqueryTableSources : v.getGroupedSubqueryTableSources()) {
                if (subqueryTableSources.size() < 2) {
                    continue;
                }
                groupedSubqueryTableSources.add(subqueryTableSources);
            }
        }

        if (groupedSubqueryTableSources.size() == 0) {
            return;
        }

        SQLTableAliasCollectVisitor v = new SQLTableAliasCollectVisitor();
        select.accept(v);


        int seed = 1;
        for (List<SQLSubqueryTableSource> tableSources : groupedSubqueryTableSources) {
            String alias = v.genAlias(seed);
            seed = v.getSeed();

            SQLSubqueryTableSource first = tableSources.get(0);

            if (with == null) {
                with = new SQLWithSubqueryClause();
                select.setWithSubQuery(with);
            }

            SQLSelect entrySelect = first.getSelect().clone();
            final SQLWithSubqueryClause.Entry withEntry = new SQLWithSubqueryClause.Entry(alias, entrySelect);
            with.addEntry(withEntry);

            for (SQLSubqueryTableSource tableSource : tableSources) {
                final SQLIdentifierExpr entryRef = new SQLIdentifierExpr(alias);
                entryRef.setResolvedTableSource(withEntry);
                SQLUtils.replaceInParent(tableSource, new SQLExprTableSource(entryRef, tableSource.getAlias()));
            }
        }

//        SQLJoinTableSource join = (SQLJoinTableSource) from;
//
//        List<SQLTableSource> joinItems = new ArrayList<SQLTableSource>();
//        join.splitTo(joinItems, join.getJoinType());
//
//        List<SQLSubqueryTableSource> subqueryJoinItems = new ArrayList<SQLSubqueryTableSource>();
//        for (SQLTableSource joinItem : joinItems) {
//            if (joinItem instanceof SQLSubqueryTableSource) {
//                final SQLSubqueryTableSource subqueryJoinItem = (SQLSubqueryTableSource) joinItem;
//                final SQLSelectQueryBlock subQueryBlock = subqueryJoinItem.getSelect().getQueryBlock();
//                if (subQueryBlock == null) {
//                    continue;
//                }
//                final SQLTableSource subFrom = subQueryBlock.getFrom();
//                if (!(subFrom instanceof SQLSubqueryTableSource)) {
//                    continue;
//                }
//                subqueryJoinItems.add((SQLSubqueryTableSource) subFrom);
//            }
//        }
//
//        final SQLSubqueryTableSource first = subqueryJoinItems.get(0);
//        SQLSubqueryTableSource firstSubFrom = (SQLSubqueryTableSource) first
//                .getSelect()
//                .getQueryBlock()
//                .getFrom();
//
//        for (int i = 1; i < subqueryJoinItems.size(); i++) {
//            final SQLSubqueryTableSource subqueryJoinItem = subqueryJoinItems.get(i);
//
//            SQLSubqueryTableSource subFrom = (SQLSubqueryTableSource) subqueryJoinItem
//                    .getSelect()
//                    .getQueryBlock()
//                    .getFrom();
//
//            if (!firstSubFrom.getSelect()
//                    .equals(
//                            subFrom.getSelect())) {
//                return;
//            }
//        }
    }

    public void visitTPCDSQ74(SQLSelectQueryBlock x) {
        final SQLObject parent = x.getParent();
        if(!(parent instanceof SQLSelect)) {
            return;
        }
        SQLSelect select = (SQLSelect) parent;

        if (!(select.getParent() instanceof SQLSelectStatement)) {
            return;
        }

        final SQLWithSubqueryClause with = select.getWithSubQuery();
        if (with == null || with.getRecursive() != null || with.getEntries().size() != 1) {
            return;
        }

        final SQLWithSubqueryClause.Entry withEntry = with.getEntries().get(0);
        final long entryAliasHashCode64 = withEntry.aliasHashCode64();

        final SQLSelectQuery withEntryQuery = withEntry.getSubQuery().getQuery();
        if (!(withEntryQuery instanceof SQLUnionQuery)) {
            return;
        }

        SQLUnionQuery withEntryUnion = (SQLUnionQuery) withEntryQuery;
        final List<SQLSelectQuery> withEntryUnionQueries = withEntryUnion.getChildren();
        for (SQLSelectQuery withEntryUnionQuery : withEntryUnionQueries) {
            if (!(withEntryUnionQuery instanceof SQLSelectQueryBlock)) {
                return;
            }
        }

        final SQLTableSource from = x.getFrom();
        if (!(from instanceof SQLJoinTableSource)) {
            return;
        }

        SQLJoinTableSource join = (SQLJoinTableSource) from;
        List<SQLTableSource> tableSources = new ArrayList<SQLTableSource>();
        join.splitTo(tableSources, join.getJoinType());

        Map<Long, SQLTableSource> tableSourceMap = new HashMap<Long, SQLTableSource>();
        for (SQLTableSource tableSource : tableSources) {
            if (!(tableSource instanceof SQLExprTableSource)) {
                return;
            }
            SQLExpr expr = ((SQLExprTableSource) tableSource).getExpr();
            if (!(expr instanceof SQLIdentifierExpr)) {
                return;
            }
            if (((SQLIdentifierExpr) expr).nameHashCode64() != entryAliasHashCode64) {
                return;
            }
            tableSourceMap.put(tableSource.aliasHashCode64(), tableSource);
        }

        Map<Long, List<SQLBinaryOpExpr>> columnCondtionMap = new HashMap<Long, List<SQLBinaryOpExpr>>();
        Map<Long, List<SQLBinaryOpExpr>> tableSourceCondtionMap = new HashMap<Long, List<SQLBinaryOpExpr>>();
        List<SQLExpr> conditions = SQLBinaryOpExpr.split(x.getWhere(), SQLBinaryOperator.BooleanAnd);
        for (SQLExpr item : conditions) {
            if (!(item instanceof SQLBinaryOpExpr)) {
                continue;
            }

            SQLBinaryOpExpr condition = (SQLBinaryOpExpr) item;
            if (ConstFolding.isLeftNameAndRightLiteral(item)) {
                if (condition.getLeft() instanceof SQLPropertyExpr) {
                    SQLPropertyExpr name = (SQLPropertyExpr) condition.getLeft();

                    if (name.getOwner() instanceof SQLIdentifierExpr) {
                        SQLIdentifierExpr owner = (SQLIdentifierExpr) name.getOwner();

                        final long nameHashCode64 = owner.nameHashCode64();
                        List<SQLBinaryOpExpr> columnConditions = tableSourceCondtionMap.get(nameHashCode64);
                        if (columnConditions == null) {
                            columnConditions = new ArrayList<SQLBinaryOpExpr>();
                            tableSourceCondtionMap.put(nameHashCode64, columnConditions);
                        }
                        columnConditions.add(condition);
                    } else {
                        return;
                    }

                    final long nameHashCode64 = name.nameHashCode64();
                    List<SQLBinaryOpExpr> columnConditions = columnCondtionMap.get(nameHashCode64);
                    if (columnConditions == null) {
                        columnConditions = new ArrayList<SQLBinaryOpExpr>();
                        columnCondtionMap.put(nameHashCode64, columnConditions);
                    }
                    columnConditions.add(condition);


                } else {
                    return;
                }
            }
        }

        Map<Long, SQLSelectQueryBlock> tableLiteralQueryMap = new HashMap<Long, SQLSelectQueryBlock>();
        Map<Long, SQLLiteralExpr> tableLiteralMap = new HashMap<Long, SQLLiteralExpr>();
        Map<Long, SQLBinaryOpExpr> tableConditionMap = new HashMap<Long, SQLBinaryOpExpr>();
        Map<SQLLiteralExpr, SQLSelectQueryBlock> LiteralQueryMap = new LinkedHashMap<SQLLiteralExpr, SQLSelectQueryBlock>();
        Map<SQLLiteralExpr, SQLWithSubqueryClause.Entry> LiteralEntryMap = new LinkedHashMap<SQLLiteralExpr, SQLWithSubqueryClause.Entry>();

        int aliasSeed = 1;
        for (Map.Entry<Long, List<SQLBinaryOpExpr>> entry : tableSourceCondtionMap.entrySet()) {
            final List<SQLBinaryOpExpr> tableConditions = entry.getValue();
            for (SQLBinaryOpExpr tableCondition : tableConditions) {
                SQLPropertyExpr name = (SQLPropertyExpr) tableCondition.getLeft();
                SQLIdentifierExpr owner = (SQLIdentifierExpr) name.getOwner();

                boolean matchLiteralColumnAllEntryQueries = true;
                SQLSelectQueryBlock matchedLiteralQuery = null;
                for (SQLSelectQuery withEntryUnionQuery : withEntryUnionQueries) {
                    SQLSelectQueryBlock selectQueryBlock = ((SQLSelectQueryBlock) withEntryUnionQuery);
                    SQLSelectItem selectItem = selectQueryBlock.findSelectItem(name.nameHashCode64());
                    if (!(selectItem.getExpr() instanceof SQLLiteralExpr)) {
                        matchLiteralColumnAllEntryQueries = false;
                        break;
                    }
                    if (selectItem.getExpr().equals(tableCondition.getRight())) {
                        matchedLiteralQuery = selectQueryBlock;
                    }
                }

                final SQLLiteralExpr right = (SQLLiteralExpr) tableCondition.getRight();
                if (matchLiteralColumnAllEntryQueries && matchedLiteralQuery != null) {
                    tableLiteralQueryMap.put(owner.nameHashCode64(), matchedLiteralQuery);
                    tableLiteralMap.put(owner.nameHashCode64(), right);
                    tableConditionMap.put(owner.nameHashCode64(), tableCondition);

                    if (!LiteralQueryMap.containsKey(right)) {
                        final SQLSelectQueryBlock literalQuery = matchedLiteralQuery.clone();
                        LiteralQueryMap.put(right, literalQuery);

                        SQLWithSubqueryClause.Entry literalEntry = new SQLWithSubqueryClause.Entry();
                        literalEntry.setSubQuery(new SQLSelect(literalQuery));
                        String withEntryAlias = withEntry.getAlias2();
                        String literalEntryAlias = null;
                        for (; aliasSeed < 100;) {
                            String alias = withEntryAlias + "_" + (aliasSeed++);
                            long aliasHashCode64 = FnvHash.hashCode64(alias);
                            if (!tableLiteralQueryMap.containsKey(aliasHashCode64)) {
                                literalEntryAlias = alias;
                                break;
                            }
                        }
                        if (literalEntryAlias == null) {
                            return;
                        }
                        literalEntry.setAlias(literalEntryAlias);
                        LiteralEntryMap.put(right, literalEntry);
                    }
                }
            }
        }

        if (LiteralEntryMap.size() != withEntryUnionQueries.size()) {
            return;
        }
        with.getEntries().clear();
        for (SQLWithSubqueryClause.Entry literalEntry : LiteralEntryMap.values()) {
            with.addEntry(literalEntry);
        }

        for (Map.Entry<Long, SQLTableSource> itemEntry : tableSourceMap.entrySet()) {
            long aliasHashCode64 = itemEntry.getKey();
            SQLExprTableSource tableSource = (SQLExprTableSource) itemEntry.getValue();
            SQLLiteralExpr literalExpr = tableLiteralMap.get(aliasHashCode64);
            String literalEntryAlias = LiteralEntryMap.get(literalExpr).getAlias();
            SQLIdentifierExpr tableExpr = new SQLIdentifierExpr(literalEntryAlias);
            tableSource.setExpr(tableExpr);
        }
        for (SQLBinaryOpExpr condition : tableConditionMap.values()) {
            SQLUtils.replaceInParent(condition, null);
        }

        tableLiteralQueryMap.size();
    }

    public void visitTPCDSQ44(SQLSelectQueryBlock x) {
        final SQLObject parent = x.getParent();
        if(!(parent instanceof SQLSelect)) {
            return;
        }
        SQLSelect select = (SQLSelect) parent;

        if (!(select.getParent() instanceof SQLSelectStatement)) {
            return;
        }

        SQLWithSubqueryClause with = select.getWithSubQuery();
        if (with != null) {
            return;
        }

        final SQLTableSource from = x.getFrom();
        if (!(from instanceof SQLJoinTableSource)) {
            return;
        }

        SQLJoinTableSource join = (SQLJoinTableSource) from;

        List<SQLTableSource> outTableSources = new ArrayList<SQLTableSource>();
        join.splitTo(outTableSources, join.getJoinType());

        List<SQLSubqueryTableSource> subqueryTableSources = new ArrayList<SQLSubqueryTableSource>();
        for (SQLTableSource tableSource : outTableSources) {
            if (!(tableSource instanceof SQLSubqueryTableSource)) {
                continue;
            }

            SQLSelectQueryBlock queryBlock = ((SQLSubqueryTableSource) tableSource).getSelect().getQueryBlock();
            if (queryBlock == null) {
                continue;
            }

            final SQLTableSource subFrom = queryBlock.getFrom();
            if (!(subFrom instanceof SQLSubqueryTableSource)) {
                continue;
            }

            SQLSelectQueryBlock subQueryBlock = ((SQLSubqueryTableSource) subFrom)
                    .getSelect()
                    .getQueryBlock();
            if (subQueryBlock == null) {
                continue;
            }

            final SQLTableSource subFrom2 = subQueryBlock.getFrom();
            if (!(subFrom2 instanceof SQLSubqueryTableSource)) {
                continue;
            }

            SQLSelectQueryBlock subQueryBlock2 = ((SQLSubqueryTableSource) subFrom2)
                    .getSelect()
                    .getQueryBlock();
            if (subQueryBlock2 == null) {
                continue;
            }

            subqueryTableSources.add(((SQLSubqueryTableSource) subFrom2));
        }

        if (subqueryTableSources.size() < 2) {
            return;
        }

        SQLSubqueryTableSource first = subqueryTableSources.get(0);
        for (int i = 1; i < subqueryTableSources.size(); ++i) {
            SQLSubqueryTableSource subqueryTableSource = subqueryTableSources.get(i);
            if (!first.getSelect()
                    .equals(
                            subqueryTableSource.getSelect())) {
                return;
            }
        }

        SQLTableAliasCollectVisitor v = new SQLTableAliasCollectVisitor();
        select.accept(v);
        String alias = v.genAlias(1);
        if (alias == null) {
            return;
        }

        SQLWithSubqueryClause.Entry withEntry = new SQLWithSubqueryClause.Entry(alias, first.getSelect().clone());
        if (with == null) {
            with = new SQLWithSubqueryClause();
            select.setWithSubQuery(with);
        }

        with.addEntry(withEntry);
        for (SQLSubqueryTableSource subqueryTableSource : subqueryTableSources) {
            SQLIdentifierExpr tableRef = new SQLIdentifierExpr(alias);
            tableRef.setResolvedTableSource(withEntry);
            SQLExprTableSource tableRefTableSource = new SQLExprTableSource(tableRef, subqueryTableSource.getAlias());
            SQLSelectQueryBlock subQueryBlock = (SQLSelectQueryBlock) subqueryTableSource.getParent();
            subQueryBlock.setFrom(tableRefTableSource);
        }
    }


    public void visitYouku01(SQLSelect select) {
        if (!(select.getParent() instanceof SQLSelectStatement)) {
            return;
        }

        final SQLSelectQueryBlock queryBlock = select.getQueryBlock();
        if (queryBlock == null) {
            return;
        }

        final SQLTableSource from = queryBlock.getFrom();
        if (!(from instanceof SQLUnionQueryTableSource)) {
            return;
        }

        final SQLUnionQuery union = ((SQLUnionQueryTableSource) from).getUnion();

        final List<SQLSelectQuery> unionSelectQueries = union.getChildren();

        if (unionSelectQueries.size() < 2) {
            return;
        }

        SQLSelectQueryBlock first = null;
        for (int i = 0; i < unionSelectQueries.size(); i++) {
            SQLSelectQuery item = unionSelectQueries.get(i);
            if (!(item instanceof SQLSelectQueryBlock)) {
                return;
            }

            final SQLTableSource unionQueryFrom = ((SQLSelectQueryBlock) item).getFrom();
            if (!(unionQueryFrom instanceof SQLExprTableSource)) {
                return;
            }
            final SQLExpr expr = ((SQLExprTableSource) unionQueryFrom).getExpr();
            if (!(expr instanceof SQLName)) {
                return;
            }

            SQLSelectQueryBlock unionQueryBlock = (SQLSelectQueryBlock) item.clone();

            final SQLSelectGroupByClause groupBy = unionQueryBlock.getGroupBy();
            if (groupBy != null) {
                boolean partitionByGroup = false;
                for (SQLExpr groupByExpr : groupBy.getItems()) {
                    if (groupByExpr instanceof MySqlOrderingExpr) {
                        groupByExpr = ((MySqlOrderingExpr) groupByExpr).getExpr();
                    }

                    if (groupByExpr instanceof SQLName) {
                        final SQLColumnDefinition resolvedColumn = ((SQLName) groupByExpr).getResolvedColumn();
                        if (resolvedColumn == null) {
                            continue;
                        }
                        if (resolvedColumn.isPartitionBy()) {
                            partitionByGroup = true;
                            break;
                        }
                    }
                }
                if (partitionByGroup) {
                    continue;
                }
            }

            unionQueryBlock.getSelectList().clear();
            unionQueryBlock.addSelectItem(new SQLIntegerExpr(1));
            unionQueryBlock.setGroupBy(null);

            if (first == null) {
                first = unionQueryBlock;
            } else if (!first.equals(unionQueryBlock)) {
                return;
            }
        }

        if (first == null) {
            return;
        }

        final Set<SQLName> columns = new LinkedHashSet<SQLName>();
        {
            SQLASTVisitorAdapter v = new SQLASTVisitorAdapter() {
                public boolean visit(SQLIdentifierExpr x) {
                    columns.add(x);
                    return false;
                }

                public boolean visit(SQLPropertyExpr x) {
                    columns.add(x);
                    return false;
                }

                public boolean visit(SQLExprTableSource x) {
                    return false;
                }

                public boolean visit(SQLSubqueryTableSource x) {
                    return false;
                }
            };
            for (SQLSelectQuery item : unionSelectQueries) {
                item.accept(v);
            }
        }

        first.getSelectList().clear();
        for (SQLName column : columns) {
            first.addSelectItem(column);
        }

        SQLWithSubqueryClause with = select.getWithSubQuery();
        if (with == null) {
            with = new SQLWithSubqueryClause();
            select.setWithSubQuery(with);
        }

        SQLTableAliasCollectVisitor v = new SQLTableAliasCollectVisitor();
        select.accept(v);
        String alias = v.genAlias(1);

        final SQLWithSubqueryClause.Entry withEntry = new SQLWithSubqueryClause.Entry(alias, new SQLSelect(first));
        with.addEntry(withEntry);

        for (int i = 0; i < unionSelectQueries.size(); i++) {
            final SQLIdentifierExpr entryRef = new SQLIdentifierExpr(alias);
            entryRef.setResolvedTableSource(withEntry);

            SQLSelectQueryBlock unionQuery = (SQLSelectQueryBlock) unionSelectQueries.get(i);
            unionQuery.setFrom(new SQLExprTableSource(entryRef));
            unionQuery.setWhere(null);
        }
    }
}
