package com.alibaba.fastsql.sql.optimizer.rules;

import com.alibaba.fastsql.sql.SQLUtils;
import com.alibaba.fastsql.sql.ast.SQLExpr;
import com.alibaba.fastsql.sql.ast.SQLName;
import com.alibaba.fastsql.sql.ast.expr.SQLIdentifierExpr;
import com.alibaba.fastsql.sql.ast.statement.*;
import com.alibaba.fastsql.sql.repository.SchemaObject;
import com.alibaba.fastsql.sql.repository.SchemaRepository;

import java.util.ArrayList;
import java.util.List;

import static com.alibaba.fastsql.util.StringUtils.quoteAlias;
import static com.alibaba.fastsql.util.StringUtils.backquoteAlias;

/**
 * @author lijun.cailj 2018/9/7
 */
public class ViewRewrite extends OptimizerVisitor {

    private List<SchemaObject> views = new ArrayList();
    private SchemaRepository repository;
    private boolean backQuotedNames = false;

    public ViewRewrite(SchemaRepository repository) {
        this.repository = repository;
    }

    public boolean visit(SQLExprTableSource x) {
        String alias = x.getAlias();
        String alias2 = backQuotedNames
                ? backquoteAlias(alias)
                : quoteAlias(alias);
        if (alias2 != alias) {
            x.setAlias(alias2);
        }

        final SQLExpr expr = x.getExpr();

        if (expr instanceof SQLName && repository != null) {
            String name = SQLUtils.normalize(((SQLName) expr).getSimpleName());

            SQLTableSource resolvedTableSource = null;
            if (expr instanceof SQLIdentifierExpr) {
                resolvedTableSource = ((SQLIdentifierExpr) expr).getResolvedTableSource();
            }

            if (resolvedTableSource == null && x.getSchemaObject() == null) {
                SchemaObject view = repository.findView(name);
                if (view != null && !views.contains(view)) {
                    views.add(view);

                    SQLSelect subQuery = ((SQLCreateViewStatement) view.getStatement()).getSubQuery();
                    subQuery.getQueryBlock().getFrom().accept(this);
                }
            }
        }

        return super.visit(x);
    }

    public void endVisit(SQLSelect x) {
        SQLWithSubqueryClause withSubQuery = x.getWithSubQuery();

        if (withSubQuery == null && views != null && views.size() > 0) {
            withSubQuery = new SQLWithSubqueryClause();
            x.setWithSubQuery(withSubQuery);
        }

        for (int i = views.size() - 1; i >= 0; i--) {
            SchemaObject view = views.get(i);

            SQLWithSubqueryClause.Entry entry = new SQLWithSubqueryClause.Entry();

            SQLSelect subQuery = ((SQLCreateViewStatement) view.getStatement()).getSubQuery();

            entry.setSubQuery(subQuery.clone());
            entry.setAlias(view.getName());
            withSubQuery.addEntry(entry);
        }
    }

    public boolean isBackQuotedNames()
    {
        return backQuotedNames;
    }

    public void setBackQuotedNames(boolean backQuotedNames)
    {
        this.backQuotedNames = backQuotedNames;
    }
}
